/*
 * IVisualContainer.java
 *
 * Created on 17. April 2000, 22:06
 */
 
package YARB.Interfaces;
/** 
 * This is the interface for a visual container, that is the container for one visualization configuration.
 * It hold specific information an a file source, data filters and a data viewer.
 * @author  Thomas
 * @version 
 */
public interface IVisualContainer extends IConfigurable 
{
  
  /**
  * Sets the file source.
  * When there is already a file source, this would be replaced.
  */
  void setFileSource(IFileSource FS);
  
  /**
  * Gets the actual file source.
  */
  IFileSource getFileSource();
  
  /**
  * Sets the data viewer.
  * When there is already a data viewer, this would be replaced.
  */
  void setDataView(IDataViewer DV);
  
  /**
  * Gets the actual data viewer.
  */
  IDataViewer getDataView();
  
  /**
  * Gets the number of contained data filters.
  */
  int numbOfDataFilter();
  
  /**
  * Sets a data filter to the specified position.
  * If there is already a data filter at the given psoition, this and all following 
  * would be psotioned one after.
  */
  void setDataFilter(int nPos, IDataFilter DF);
  
  /**
  * Adds a data filter to the end.
  */
  void addDataFilter(IDataFilter DF);
  /**
  * Deletes the data filter a the given position.
  */
  void deleteDataFilter(int nPos);
  
  /**
  * Gets the data filter at the given position.
  */
  IDataFilter getDataFilter(int nPos);
  
  /**
  * Sets the file name for the data file to be visualized.
  * This file name will be given to the file source, and also to the data viewer for proprietary viewing.
  */
  void setFile(String strFile);
  
  /** 
  * Starts the process of visualization
  */
  boolean DoVisualization(IMainFrame ParentFrame)  throws IDataProvider.DataProcessingError;
 
}
