/*
 * YDefaultDataHolder.java
 *
 * Created on 9. Mai 2000, 14:31
 */
 
package YARB.Main;
import YARB.Interfaces.*;
import java.util.ArrayList;
/** 
 *
 * @author  Thomas
 * @version 
 */
public class YDefaultDataHolder 
extends Object 
implements IDataHolder
{

  float      m_InBuffer[][];
  float      m_OutBuffer[][];
  int        m_nNumbOfPattern;
  int        m_nNumbOfInUnits;
  int        m_nNumbOfOutUnits;
   
  int        m_VarInDimension;
  int        m_MaxVarInDim[];
  int        m_VarOutDimension;
  int        m_MaxVarOutDim[];

  /** Creates new YDefaultDataHolder */
  public YDefaultDataHolder() {
    m_InBuffer=null;
    m_OutBuffer=null;
    m_MaxVarInDim=null;
    m_MaxVarOutDim=null;
    m_nNumbOfPattern=0;
    m_nNumbOfInUnits=0;
    m_nNumbOfOutUnits=0;
    m_VarInDimension=0;
    m_VarOutDimension=0;
  }
  
  /** Sets the data provider.
  * This function copies all the data from the given data provider into 
  * its own buffer. It does NOT call the ProcessData and FreeData functions of the provider!
  */
  public void setData(IDataProvider DP) throws IDataProvider.DataProcessingError 
  {
    
    if( DP==null) return;
    
    // create the new Array of Pattern
    m_InBuffer= new float[DP.numbOfPatterns()][DP.numbOfInputUnits()];
    m_OutBuffer= new float[DP.numbOfPatterns()][DP.numbOfOutputUnits()];
    
    // Copy all Pattern
    for(int i =0; i < DP.numbOfPatterns(); i++)
    {
      // Copy Input pattern
      for(int k=0; k< DP.numbOfInputUnits(); k++)
       {
         m_InBuffer[i][k]=DP.getInputData(i, k);
       };
       
      // Copy Output Pattern
      for(int k=0; k< DP.numbOfOutputUnits(); k++)
       {
         m_OutBuffer[i][k]=DP.getOutputData(i, k);
       };
    };
      
   // Copy state information:
  
    m_nNumbOfPattern=DP.numbOfPatterns();
    m_nNumbOfInUnits=DP.numbOfInputUnits();
    m_nNumbOfOutUnits=DP.numbOfOutputUnits();
    m_VarInDimension= DP.numbOfVariableInputDimensions();
    m_VarOutDimension= DP.numbOfVariableOutputDimensions();
      
    if(m_VarInDimension>0)
    {
        m_MaxVarInDim= new int[m_VarInDimension];
        for(int j=0; j<m_VarInDimension;j++)
        {
          m_MaxVarInDim[j]=DP.getMaxInputDimesion(j);
        };
    };
      
    if(m_VarOutDimension>0)
    {
        m_MaxVarOutDim= new int[m_VarOutDimension];
        for(int j=0; j<m_VarOutDimension;j++)
        {
          m_MaxVarOutDim[j]=DP.getMaxOutputDimesion(j);
        };
    };
  }
  
  /** Gets the number of pattern.
   */
  public int numbOfPatterns() {
    return m_nNumbOfPattern;
  }
  
  /** Gets the number of input units.
   */
  public int numbOfInputUnits() {
    return m_nNumbOfInUnits;
  }
  
  /** Gets the number of Output Units
   */
  public int numbOfOutputUnits() {
    return m_nNumbOfOutUnits;
  }
  
  /** Gets the number of variable Input dimensions
   */
  public int numbOfVariableInputDimensions() {
    return m_VarInDimension;
  }
  
  /** Gets the number of variable Output dimensions
   */
  public int numbOfVariableOutputDimensions() {
    return m_VarOutDimension;
  }
  
  /** Gets the maximum input dimensions range for the given variable input dimension
  */
  public int getMaxInputDimesion(int nVarInputDimension) 
  {
    // Check array bounds:
    if(nVarInputDimension>=m_VarInDimension || nVarInputDimension < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getMaxInputDimension(int); Index: " + nVarInputDimension + "; Max: " + m_VarInDimension + ";)";
      System.out.println(msg);
      return 0;
    };
    // Everything O.K.
    return m_MaxVarInDim[nVarInputDimension];
  }
  
  /** Gets the maximum output dimensions range for the given variable output dimension
   */
  public int getMaxOutputDimesion(int nVarOutputDimension)
  {
    // Check array bounds:
    if(nVarOutputDimension>=m_VarOutDimension || nVarOutputDimension < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getMaxOutputDimension(int); Index: " + nVarOutputDimension + "; Max: " + m_VarOutDimension + ";)";
      System.out.println(msg);
      return 0;
    };
    // Everything O.K.
    return m_MaxVarOutDim[nVarOutputDimension];
  }
  
  /** Gets a input data field, inthe given pattern and specified by the give unit.
   */
  public float getInputData(int nPattern,int nUnit) 
  {
    // Check array Bounds:
    if(nPattern>=m_nNumbOfPattern || nPattern < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getInputData(int,int); Index: " + nPattern + "; Max: " + m_nNumbOfPattern + ";)";
      System.out.println(msg);
      return 0;
    };
    if(nUnit>=m_nNumbOfInUnits || nUnit < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getInputData(int,int); Index: " + nUnit + "; Max: " + m_nNumbOfInUnits + ";)";
      System.out.println(msg);
      return 0;
    };
    // Bounds OK, return value
    return m_InBuffer[nPattern][nUnit];
  }
  
  
  /** Gets a output data field, inthe given pattern and specified by the give unit.
   */
  public float getOutputData(int nPattern,int nUnit) 
  {
    // Check array Bounds:
    if(nPattern>=m_nNumbOfPattern || nPattern < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getOutputData(int,int); Index: " + nPattern + "; Max: " + m_nNumbOfPattern + ";)";
      System.out.println(msg);
      return 0;
    };
    if(nUnit>=m_nNumbOfOutUnits || nUnit < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getOutputData(int,int); Index: " + nUnit + "; Max: " + m_nNumbOfOutUnits + ";)";
      System.out.println(msg);
      return 0;
    };
    // Bounds OK, return value
    return m_OutBuffer[nPattern][nUnit];
  }
}