/*
 * GlobalNavigationDirector.java
 *
 * Created on 19. April 2000, 15:13
 */
 
package YARB.Main;
import YARB.Interfaces.*;
import YARB.Dialogs.*;
import YARB.XML.*;
import javax.swing.*;
import java.awt.Cursor;
import java.awt.event.MouseAdapter;
import java.util.*;
import java.io.File;
import javax.swing.filechooser.*;


/** 
 *
 * @author  Thomas
 * @version 
 */
public class YGlobalNavigationDirector extends Object {

  /** The one and only GlogalNavigationDirector onject */
  private static YGlobalNavigationDirector m_Exemplar=null;
  
  public static final int OT_UNKNOWN=0;
  public static final int OT_ICONTAINER=1;
  public static final int OT_IFILE=2;
  public static final int OT_IFILTER=3;
  public static final int OT_IVIEWER=4;
  
  private IConfigurable m_Object;
  private int           m_ObjType;
  
  private YVNavigator   m_VNavigator;
  private YMainFrame    m_MainFrame;
  
  private File m_strLastPath;
  
  /**
   * Private constructor prevents form being created outside.
   */
  private YGlobalNavigationDirector() 
  {
    m_Object=null;
    m_ObjType=OT_UNKNOWN;    
    m_VNavigator=null;
    m_MainFrame=null;
    m_strLastPath=null;
  }
  
   /**
   * This function returns the one and only object.
   * The object is created on demand.
   */
  static YGlobalNavigationDirector exemplar()
  {
    if(m_Exemplar==null)
    {
      m_Exemplar= new YGlobalNavigationDirector();
    };
    
    return m_Exemplar;
  }

  
  /** 
  * Set a new navigatior frame 
  */
  void setVNavigator(YVNavigator VNav){
    m_VNavigator=VNav;
  }
    
  /**
  * Set a new main frame
  */
  void setMainFrame(YMainFrame MF){
    m_MainFrame=MF;
  }
  
  /** 
  * Sets the active object and determines the corresponding class
  */
  void setActiveObject(IConfigurable o)
  {
    setActiveObject(o,getObjectType(o));

  }
  
   /**
   * Determines the class of the given object
   */
  int getObjectType(IConfigurable o)
  {
      if(o==null) {
        return OT_UNKNOWN;
      }
      
      Class[] theInterfaces = o.getClass().getInterfaces();
      for (int i = 0; i < theInterfaces.length; i++) 
      {
        String str=theInterfaces[i].getName();
        if( str.equals("YARB.Interfaces.IVisualContainer")){
           return OT_ICONTAINER;                  
        };
        if( str.equals("YARB.Interfaces.IFileSource")) {
           return OT_IFILE;
        };
        if( str.equals("YARB.Interfaces.IDataFilter")) {
           return OT_IFILTER;                  
         };
        if( str.equals("YARB.Interfaces.IDataViewer")){
           return OT_IVIEWER;
        };
      }
        
      return OT_UNKNOWN;
  }
  
  /**
   *  Sets the active object and type
   */
  void setActiveObject(IConfigurable o, int nObjType)
  {
    m_Object=o;
    if( o == null)
    {
      m_ObjType=OT_UNKNOWN;
    }
    else
    {
      m_ObjType=nObjType;
    }
    
    // Update menu and toolbar

    UpdateAllMenusToolbars();
  
  }
  
  /**
  * Returns the active object
  */
  IConfigurable getActiveObject(){
    return m_Object;
  }
  
  /**
  * Returns the active Object Type
  */
  int getActiveObjectType(){
    return m_ObjType;
  }
  
  
  /** 
  * Updates the assiciated views
  */
  void UpdateAllViews()
  {
    if(m_VNavigator!=null)
    {
      m_VNavigator.PopulateTree();
      m_VNavigator.setVisibleObject(m_Object);
    };
  }    
  
  /** 
  * Updates the assiciated views
  */
  void UpdateAllViews(IConfigurable o)
  {
    if(m_VNavigator!=null)
    {
      m_VNavigator.PopulateTree();
      m_VNavigator.setVisibleObject(o);
    };
  }  
  
  /**
  * Updates all Menus and Toolbars
  */
  void UpdateAllMenusToolbars()
  {
      if(m_MainFrame!=null)
    {
      m_MainFrame.updateMenu();
      m_MainFrame.updateToolbar();
    }
  }
  
  /**
  * Query whether a new container could be added or not
  */
  boolean canAddNewContainer(){
    return true;
  }

  /**
  * Adds a new Container. Takes the default container.
  */
  boolean addNewContainer()
  {
   if(!canAddNewContainer())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't create a new container for the selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };

    YVisualizationManager man= YVisualizationManager.exemplar();
    IConfigurable o= man.createContainer(null);
    if( null == o){
      JOptionPane.showMessageDialog(m_MainFrame,"Error creating container.","YARB Error",JOptionPane.ERROR_MESSAGE);
      return false;
    };
    
    UpdateAllViews(o);
    
    return true;
  }

  /**
  * Query whether a new file source could be added or not
  */
  boolean canAddNewFileSource(){
    return m_ObjType!= OT_UNKNOWN;
  }

  /**
  * Asks the user for a file source type, creates a new one and binds it to the actual container.
  */
  boolean addNewFileSource()
  {
    if(!canAddNewFileSource())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't create a new file source for the selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YVisualizationManager man= YVisualizationManager.exemplar();
    String [][] a= man.getAvailableFileSources();
    IVisualContainer cont= man.getContainerByObject(m_Object);
    
    if(cont==null)
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Couldn't find configuration for selected iten.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YClassChooser dlg = new YClassChooser(m_MainFrame,"Please choose a new file source",true);
    
    dlg.setInterface("YARB.Interfaces.IFileSource");
    dlg.setClasses(a);
    
    dlg.show();
    
    // get the class name
    String fs=dlg.getSelectedClass();

    if(fs==null)  // User chancel?
    {
      return true;
    };
    
    // create the object and bind it to the container
    IFileSource o= man.createFileSource(fs);
    if( null == o){
      JOptionPane.showMessageDialog(m_MainFrame,"Error creating file source.","YARB Error",JOptionPane.ERROR_MESSAGE);
      return false;
    };
    
    cont.setFileSource(o);
    UpdateAllViews(o);

    return true;
  }
  
  /**
  * Query whether a new data filter could be added or not
  */
  boolean canAddNewDataFilter(){
    return m_ObjType!= OT_UNKNOWN;
  }

  /**
  * Asks the user for a data filter type, creates a new one and binds it to the actual container.
  */
  boolean addNewDataFilter()
  {
    if(!canAddNewDataFilter())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't create a new data filter for the selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YVisualizationManager man= YVisualizationManager.exemplar();
    String [][] a= man.getAvailableDataFilter();
    Vector v= new Vector();
    IVisualContainer cont= man.getContainerByObject(m_Object);
    int i;
    
    if(cont==null)
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Couldn't find configuration for selected iten.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YClassChooser dlg = new YClassChooser(m_MainFrame,"Please choose a new data filter",true);
    
    dlg.setInterface("YARB.Interfaces.IDataFilter");
    dlg.setClasses(a);
    
    dlg.show();
    
    // get the class name
    String fs=dlg.getSelectedClass();

    if(fs==null)  // User chancel?
    {
      return true;
    };
    
    // create the object and bind it to the container
    IDataFilter o= man.createDataFilter(fs);
    if( null == o){
      JOptionPane.showMessageDialog(m_MainFrame,"Error creating data filter.","YARB Error",JOptionPane.ERROR_MESSAGE);
      return false;
    };
    
    cont.addDataFilter(o);
    UpdateAllViews(o);
    
    return true;
  }
    
  
  /**
  * Query whether a new data view could be added or not
  */
  boolean canAddNewDataView(){
    return m_ObjType!= OT_UNKNOWN;
  }

  
   /**
  * Asks the user for a data view type, creates a new one and binds it to the actual container.
  */
 boolean addNewDataView()
  {
    if(!canAddNewDataView())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't create a new data view for the selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YVisualizationManager man= YVisualizationManager.exemplar();
    String [][] a= man.getAvailableDataViewer();
    Vector v= new Vector();
    IVisualContainer cont= man.getContainerByObject(m_Object);
    int i;
    
    if(cont==null)
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Couldn't find configuration for selected iten.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YClassChooser dlg = new YClassChooser(m_MainFrame,"Please choose a new data viewer",true);
    
    dlg.setInterface("YARB.Interfaces.IDataViewer");
    dlg.setClasses(a);
    
    dlg.show();
    
    // get the class name
    String fs=dlg.getSelectedClass();

    if(fs==null)  // User chancel?
    {
      return true;
    };

    // create the object and bind it to the container
    IDataViewer o= man.createDataViewer(fs);
    if( null == o){
      JOptionPane.showMessageDialog(m_MainFrame,"Error creating data view.","YARB Error",JOptionPane.ERROR_MESSAGE);
      return false;
    };
    
    cont.setDataView(o);
    UpdateAllViews(o);
    
    return true;
  }
  
  
  /**
  * Query whether the actual item could be deleted or not
  */
  boolean canDelete(){
    return m_ObjType!= OT_UNKNOWN;
  }
  
  /**
  * Deletes the acutal item
  */
  boolean doDelete()
  {
    if(!canDelete())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't delete selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YVisualizationManager man= YVisualizationManager.exemplar();
    IVisualContainer cont=man.getContainerByObject(m_Object);
    if(cont==null)
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Couldn't find configuration for selected iten.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    switch(m_ObjType)
    {
      case OT_ICONTAINER:
                          for(int i=0; i<man.getNumbVContainers(); i++)
                          {
                            if(man.getVContainer(i)==m_Object){
                              man.deleteContainer(i);
                            };
                          };
                          
                          break;
                         
      case OT_IFILTER:
                          for(int i=0; i<cont.numbOfDataFilter(); i++)
                          {
                            if(cont.getDataFilter(i)==m_Object){
                              cont.deleteDataFilter(i);
                            };
                          };
                          
                          break;
                          
      case OT_IFILE:
                          cont.setFileSource(null);
                          break;
                          
      case OT_IVIEWER:
                          cont.setDataView(null);
                          break;
    };
                          
    setActiveObject(null);                      
    UpdateAllViews(cont);

    return true;
  }
  /**
  * Query whether the actual item is movable or not
  */
  boolean canMove(){
    return m_ObjType == OT_IFILTER;
  }

  
  /**
  * Moves the current item one up
  */
  boolean doMoveUp()
  {
    
    if(!canMove())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't move selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YVisualizationManager man= YVisualizationManager.exemplar();
    IVisualContainer cont=man.getContainerByObject(m_Object);
    if(cont==null)
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Couldn't find configuration for selected iten.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };


    // search my filter:
    
    int pos;
    boolean bFound=false;
    
    for(pos=0;pos<cont.numbOfDataFilter();pos++)
    {
      if(cont.getDataFilter(pos)==m_Object)
      {
        bFound=true;
        break;
      };
    };
    
    if(!bFound){
      return false;
    };
    
    // Are we on top?
    if(pos==0){
      return false;
    };
    
    // Move:
    cont.setDataFilter(pos-1,(IDataFilter)m_Object);
    cont.deleteDataFilter(pos+1);
 
    UpdateAllViews();
    
    return true;
  }
  

  /**
  * Moves the current item one down
  */
  boolean doMoveDown()
  {
    
    if(!canMove())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't move selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YVisualizationManager man= YVisualizationManager.exemplar();
    IVisualContainer cont=man.getContainerByObject(m_Object);
    if(cont==null)
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Couldn't find configuration for selected iten.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };


    // search my filter:
    
    int pos;
    boolean bFound=false;
    
    for(pos=0;pos<cont.numbOfDataFilter();pos++)
    {
      if(cont.getDataFilter(pos)==m_Object)
      {
        bFound=true;
        break;
      };
    };
    
    if(!bFound){
      return false;
    };
    
    // Are we on bottom?
    if(pos==(cont.numbOfDataFilter()-1)){
      return false;
    };
    
    // Move:
    cont.deleteDataFilter(pos);
    cont.setDataFilter(pos+1,(IDataFilter)m_Object);
 
    UpdateAllViews();
    
    return true;
  }
  

  /**
  * Query whether the actual item is configurable or not
  */
  boolean canConfigure(){
    if(m_ObjType== OT_UNKNOWN || m_Object==null){
      return false;
    };
    return(m_Object.IsConfigurable());
  }

  
  /**
  *Shows the configuration dialog of the current item
  */
  boolean doConfigure()
  {
    
    if(!canConfigure())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't configure the selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    if(!m_Object.ShowConfigurationDialog(m_MainFrame))
    {
      JOptionPane.showMessageDialog(m_MainFrame,"Error during configuration of the selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
      return false;
    };
    

    UpdateAllViews(m_Object);
    
    return true;
    
  }

  /**
  * Query whether a new Visualization could be made or not.
  */
  boolean canCreateNewVisualization(){
    if(m_ObjType== OT_UNKNOWN || m_Object==null){
      return false;
    };
    
    return true;
  }
  
  void setWaitCursor()
  {
    if(m_MainFrame!=null)
    {
      m_MainFrame.setWaitCursor();
    };
  }

  void deleteWaitCursor()
  {
    if(m_MainFrame!=null)
    {
      m_MainFrame.deleteWaitCursor();
    };
  }
      
  
  /**
  *  Creates a new visualization from the current configuration
  */
  boolean doCreateNewVisualization()
  {
    if(!canCreateNewVisualization())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't create a new visualization from the selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YVisualizationManager man= YVisualizationManager.exemplar();
    IVisualContainer cont=man.getContainerByObject(m_Object);
    if(cont==null)
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Couldn't find configuration for selected iten.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    // Ask the User for the next file:    
    JFileChooser fc = new JFileChooser();
    fc.setCurrentDirectory(m_strLastPath);
    String img[]={"jpg","jpeg","gif"};
    fc.addChoosableFileFilter(new GenericFileFilter("Just images (*.jpg, *.jpeg, *.gif)",img));
    String res[]={"res"};
    fc.addChoosableFileFilter(new GenericFileFilter("SNNS Result files (*.res)",res));
    String pat[]={"pat"};
    fc.addChoosableFileFilter(new GenericFileFilter("SNNS Pattern files (*.pat)",pat));
    
    
    int returnVal = fc.showOpenDialog(m_MainFrame);
 
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      File file = fc.getSelectedFile();
      cont.setFile(file.getPath());
      m_strLastPath=file;
    }
    else // file chooer canceled
    {
      return false;
    };

    // Now do the visualization:
    setWaitCursor();
    boolean bReturn=true;
    try
    {
      bReturn= cont.DoVisualization(m_MainFrame);
    }
    catch(IDataProvider.DataProcessingError e)
    { 
      JOptionPane.showMessageDialog(m_MainFrame,"There occured a critical error while pipline processing.\n\nERROR: " + e.getMessage(),"YARB Pipeline Error",JOptionPane.ERROR_MESSAGE);
    };
    
    if(!bReturn)
    {
      JOptionPane.showMessageDialog(m_MainFrame,"There occured an error while pipline processing.\n\nPlease check the console output for further explanation.\n\n(NOTE: The displayed data may be corrupt!)","YARB Pipeline Error",JOptionPane.ERROR_MESSAGE);
    };
    
    deleteWaitCursor();
    return true;
  }
    
    
  /**
  *  Creates a new visualization from the current configuration with the last open file
  */
  boolean doRedoCurrentVisualization()
  {
    if(!canCreateNewVisualization())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't create a new visualization from the selected item.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YVisualizationManager man= YVisualizationManager.exemplar();
    IVisualContainer cont=man.getContainerByObject(m_Object);
    if(cont==null)
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Couldn't find configuration for selected iten.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };

    
   // Now do the visualization:
    setWaitCursor();
    boolean bReturn=true;
    try
    {
      bReturn= cont.DoVisualization(m_MainFrame);
    }
    catch(IDataProvider.DataProcessingError e)
    { 
      JOptionPane.showMessageDialog(m_MainFrame,"There occured a critical error while pipline processing.\n\nERROR: " + e.getMessage(),"YARB Pipeline Error",JOptionPane.ERROR_MESSAGE);
    };
    
    if(!bReturn)
    {
      JOptionPane.showMessageDialog(m_MainFrame,"There occured an error while pipline processing.\n\nPlease check the console output for further explanation.\n\n(NOTE: The displayed data may be corrupt!)","YARB Pipeline Error",JOptionPane.ERROR_MESSAGE);
    };
    
    deleteWaitCursor();
    return true;
  }

  /**
  * Query whether a configuration could be opened or not
  */
  boolean canOpenConfiguration(){
    return true;
  }

  /**
  * Opens a new configuration
  */
  boolean doOpenConfiguration(){
    
    if(!canOpenConfiguration())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't open configration now.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YVisualizationManager man= YVisualizationManager.exemplar();
    
    // Ask the User for the next file:    
    JFileChooser fc = new JFileChooser();
    fc.setCurrentDirectory(m_strLastPath);
    String xml[]={"xml"};
    fc.addChoosableFileFilter(new GenericFileFilter("YARB XML Configuration files  (*.xml)",xml));

    int returnVal = fc.showOpenDialog(m_MainFrame);
 
   File file;
   if (returnVal == JFileChooser.APPROVE_OPTION) {
      file = fc.getSelectedFile();
      m_strLastPath=file;
    }
    else // file chooer canceled
    {
      return false;
    };

    setWaitCursor();
    
    // New reader:
    YXMLConfigurationReader Reader= new YXMLConfigurationReader();
    try
    {
      Reader.Open(file.getPath());
    }
    catch(ICfgFileRead.ECfgFileReadError e)
    {
      JOptionPane.showMessageDialog(m_MainFrame,"Error opening configuration file.\n\nERROR:" + e.toString(),"YARB File Error",JOptionPane.ERROR_MESSAGE);
      deleteWaitCursor();
      return false;
    };
      
    
    // Now open cfg:
    IConfigurable o= man.createContainer(null);
    if( null == o){
      JOptionPane.showMessageDialog(m_MainFrame,"Error creating container.","YARB Error",JOptionPane.ERROR_MESSAGE);
      deleteWaitCursor();
      return false;
    };

    
    try
    {
      o.LoadConfiguration(Reader.getReader());
    }
    catch(IConfigurationReader.ECfgReaderError e)
    {
      JOptionPane.showMessageDialog(m_MainFrame,"Error reading configuration file.\n\nERROR:" + e.toString(),"YARB File Error",JOptionPane.ERROR_MESSAGE);
      deleteWaitCursor();
      return false;
    };
    
    Reader.Close();
    
    UpdateAllViews(o);
    
    deleteWaitCursor();
    return true;
   }
    

  /**
  * Query whether the current configuration could be saved or not
  */
  boolean canSaveConfiguration(){
    if(m_ObjType== OT_UNKNOWN || m_Object==null){
      return false;
    };
    
    return true;
  }
  
  /**
  * Saves the current configuration to a file
  */
  boolean doSaveConfiguration(){
    if(!canSaveConfiguration())
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Can't save configration for the current component.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };
    
    YVisualizationManager man= YVisualizationManager.exemplar();
    
    // Ask the User for the next file:    
    JFileChooser fc = new JFileChooser();
    fc.setCurrentDirectory(m_strLastPath);
    String xml[]={"xml"};
    fc.addChoosableFileFilter(new GenericFileFilter("YARB XML Configuration files  (*.xml)",xml));

    int returnVal = fc.showSaveDialog(m_MainFrame);
 
   File file;
   if (returnVal == JFileChooser.APPROVE_OPTION) {
      file = fc.getSelectedFile();
      m_strLastPath=file;
    }
    else // file chooer canceled
    {
      return false;
    };

    IVisualContainer cont=man.getContainerByObject(m_Object);
    if(cont==null)
    {
        JOptionPane.showMessageDialog(m_MainFrame,"Couldn't find configuration for selected iten.","YARB application logic warning",JOptionPane.WARNING_MESSAGE);
        return false;
    };

     setWaitCursor();
    
    // New reader:
    YXMLConfigurationWriter Writer= new YXMLConfigurationWriter();
    Writer.Open(file.getPath());
      

    try
    {
      cont.SaveConfiguration(Writer.getWriter());
    }
    catch(IConfigurationWriter.ECfgWriterError e)
    {
      JOptionPane.showMessageDialog(m_MainFrame,"Error reading configuration file.\n\nERROR:" + e.toString(),"YARB File Error",JOptionPane.ERROR_MESSAGE);
      deleteWaitCursor();
      return false;
    };
    
    try
    {
      Writer.Close();
    }
    catch(ICfgFileWrite.ECfgFileWriteError e)
    {
      JOptionPane.showMessageDialog(m_MainFrame,"Error writing configuration file.\n\nERROR:" + e.toString(),"YARB File Error",JOptionPane.ERROR_MESSAGE);
      deleteWaitCursor();
      return false;
    };

    deleteWaitCursor();
    
    return true;
  }
  
  
  private class GenericFileFilter extends FileFilter 
  {
    String  m_strTitle;
    String  m_strExtensions[];

    GenericFileFilter(String strTitle, String strExtensions[])
    {
      m_strTitle=strTitle;
      m_strExtensions=strExtensions;
    }

    /*
     * Get the extension of a file.
     */  
    public  String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');
        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }

    // Accept all directories and all specified files
    public boolean accept(File f) {
        if (f.isDirectory()) {
            return true;
        };

        String extension = getExtension(f);
        if(m_strExtensions!=null && extension!=null)
        {
          for(int i=0; i< m_strExtensions.length; i++)
          {
            if(extension.equals(m_strExtensions[i])){
              return true;
            };
          };
        };
        return false;
    }
    // The description of this filter
    public String getDescription() {
        return m_strTitle;
    }
  }

}