/*
 * YXMLReader.java
 *
 * Created on 3. Juni 2000, 14:43
 */
 
package YARB.XML;


import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.DocumentBuilder;

import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;

import com.sun.xml.tree.XmlDocument;

import org.w3c.dom.*;

import java.io.File;
import java.io.IOException;

/** 
 *
 * @author  Administrator
 * @version 
 */
public class YXMLReader extends Object {

  // SAX Document
  private   Document document = null;
  // Whole DOM
  private   Node DomNode = null;
  // Source file
  private String m_XMLSourceFile = null;
  // Current processed node
  private Node m_CurrentNode = null;
  // Current open Node
  private String m_sCurrentOpenNode = null;
  
  //  Visited Nodes
  private NodeList m_VisitedNodes = null;
  private boolean[] m_bVisitedIndex = null;
  // Root nodes
  private NodeList m_RootNodes = null;
  private boolean[] m_bRootIndex = null;
  
  static final String[] m_typeName = {
    "none", "Element", "Attr", "Text",
    "CDATA","EntityRef", "Entity", "ProcInstr",
    "Comment", "Document", "DocType", "DocFragment",
    "Notation",
  };  
 
  // Valid Elements in XML File
  static final String[] m_validElements = {
    //"YARBMultipleConfiguration", "YARBSingleConfiguration",
    "YARBConfiguration",
    "DataFilter", "FileSources", "DataViewer",
    "Title", "Class", "File",
    "boolean", "int", "float", "String",
  };
  // Amount of valid elements in XML File
  static final int m_IvalidElements = 11;
    
  /** Creates new YXMLReader */
  public YXMLReader() {        
  }
  
  /** Set XML Source File */
  public void SetXMLSource(String src) {
    m_XMLSourceFile = src;
  }
  
  /** Set current active node */
  public void SetCurrentNode(Node node) throws ECfgReaderCGeneralError {
    m_CurrentNode = node;
    m_sCurrentOpenNode = node.getNodeName();
    if( m_CurrentNode.hasChildNodes() ) {      
      m_VisitedNodes = m_CurrentNode.getChildNodes();      
      m_bVisitedIndex = new boolean[m_VisitedNodes.getLength()];
      for( int i=0; i < m_VisitedNodes.getLength(); i++ ) m_bVisitedIndex[i] = false;
    }        
    // no childs -> error
    else {
      m_VisitedNodes = null;
      throw new ECfgReaderCGeneralError("Reached Element has no childs ! " + "\n" + this.toString());
    }
  }
  /** Returns the current node */
  public Node GetCurrentNode() {
    return m_CurrentNode;
  }
  
  /** Resets all */
  public void ResetAll() {
    // SAX Document
    document = null;
    // Whole DOM
    DomNode = null;
    // Source file
    m_XMLSourceFile = null;
    // Root node list
    m_RootNodes = null;
    m_bRootIndex = null;
    // Current processed node
    m_CurrentNode = null;
    m_sCurrentOpenNode = null;
    //  Visited Nodes
    m_VisitedNodes = null;
    m_bVisitedIndex = null;    
  }
    
  /** Resets current node */
  public void ResetNode() {
    // SAX Document
    //document = null;
    // Whole DOM
    //DomNode = null;
    // Source file
    //m_XMLSourceFile = null;
    // Current processed node
    m_CurrentNode = null;
    m_sCurrentOpenNode = null;
    //  Visited Nodes
    m_VisitedNodes = null;
    m_bVisitedIndex = null;    
  }
  
  /** Search node for specified type and name */
  private Node m_SearchResult = null;
  private boolean m_bNodeFound = false;
  
  private void SearchNode(Node node, String NodeType, String NodeName) {            
    // valid type ?
    if( NodeType.compareTo(m_typeName[node.getNodeType()]) == 0 ) {
      // valid name ?
      if( NodeName.compareTo(node.getNodeName()) == 0 ) {                
          m_SearchResult = node;
          m_bNodeFound = true;
          return ;        
      }
    }    
    if( node.hasChildNodes() ) {
      NodeList nodeList = node.getChildNodes();      
      for( int i=0; i < nodeList.getLength() ; i++) {
        SearchNode(nodeList.item(i),NodeType,NodeName);
        if( m_bNodeFound ) return;
      }      
    }    
  }
 
  /** Search for Node in nodelist */
  private Node  FindNodeList(NodeList nodeList,String NodeName, boolean[] Index) throws ECfgReaderCGeneralError {
    Node tmp;
    for( int i=0; i < nodeList.getLength(); i++) {
      tmp = nodeList.item(i);
      if( "Element".compareTo(m_typeName[tmp.getNodeType()]) == 0 ) {        
        if( NodeName.compareTo(tmp.getNodeName()) == 0 ) {
          // check if node was already processed
          if( !Index[i]  ) {
            Index[i] = true;
            return tmp;
          }          
        }
      }
    }
    // Nothing found or already processed
    throw new ECfgReaderCGeneralError("Element " + NodeName + " wasn't found or already processed! " + "\n" + "\n" + this.toString());
  }
  
  /** Search for a specified node in the DOM tree 
  * if null is specified the whole tree is used
  */
  public Node  FindNode(String NodeName) throws ECfgReaderCGeneralError {
    // Control if NodeName is a valid element
    boolean ElementFound = false;
    for( int i=0 ; i < m_IvalidElements; i++) {
      if( NodeName.compareTo(m_validElements[i]) == 0 ) ElementFound = true;
    }
    if( !ElementFound ) {
      throw new ECfgReaderCGeneralError("Element " + NodeName + " isn't valid ! " + "\n" + this.toString() );
    }
    if( document == null ) {
      throw new ECfgReaderCGeneralError("No file was opened yet ! " + "\n" + this.toString() );
    }
    try {
      // start root index
      if( NodeName.compareTo("YARBConfiguration") == 0 ) {//|| ( m_sCurrentOpenNode.compareTo("YARBConfiguration") == 0 && 0 == NodeName.compareTo("Title") ) ) {      
        //ParseXML();
        m_bNodeFound = false;
        m_SearchResult = null;      
        SearchNode(DomNode, "Element", NodeName);    
        // Check if anything was found
        if( !m_bNodeFound ) {
          throw new ECfgReaderCGeneralError("Element " + NodeName + " wasn't found ! " + "\n" + this.toString());
        }
        if( !m_SearchResult.hasChildNodes() ) throw new ECfgReaderCGeneralError("Root Element " + NodeName + " has no Elements! " + "\n" + this.toString());        
        m_RootNodes = m_SearchResult.getChildNodes();
        m_bRootIndex = new boolean[m_RootNodes.getLength()];              
        return m_SearchResult;
      }
      else {
        //if( m_sCurrentOpenNode.compareTo("YARBConfiguration") == 0) {
        if( NodeName.compareTo("FileSources") == 0  || NodeName.compareTo("DataFilter") == 0 || NodeName.compareTo("DataViewer") == 0 ) {          
          return FindNodeList(m_RootNodes, NodeName, m_bRootIndex );          
        }      
        return FindNodeList(m_VisitedNodes, NodeName, m_bVisitedIndex );          
      }    
    } catch (ECfgReaderCGeneralError e) {
      throw new ECfgReaderCGeneralError( e.toString() + " "+"\n" + this.toString() );
    }
    
  }
    
 
  /** Get Element with ElementName in current node */
  public String getElement(String ElementName) throws ECfgReaderCGeneralError {
    if( m_CurrentNode == null ) {
      throw new ECfgReaderCGeneralError("No node was yet opened ! " + "\n" + this.toString() );
    }
    Node node;
    try {
      node = FindNode(ElementName);
    } catch (ECfgReaderCGeneralError e) {
      throw new ECfgReaderCGeneralError( e.toString() +" "+ "\n" + this.toString() );
    }
    node = node.getFirstChild();
    if( node == null ) {
      throw new ECfgReaderCGeneralError("Element " + ElementName + " has no data ! " + "\n" + this.toString());
    }        
    return node.getNodeValue();
  }
  
  /** print a Dom tree to system from the given node */
  private int depth = 0;
  public void printDOMTree(org.w3c.dom.Node node) {        
    if( node == null ) return;
    
    String NodeType = m_typeName[node.getNodeType()];
    String NodeName = node.getNodeName();
    String NodeValue = node.getNodeValue();             
    for( int i=0; i <= depth; i++ ) System.out.print("  ");
    if( NodeType != null )     
      System.out.print("NodeType: " + NodeType );    
    if( NodeName != null )       
      System.out.print(" NodeName: " + NodeName );    
    if( NodeValue != null )  
      System.out.print(" NodeValue: " + NodeValue );    
    
    System.out.println("");
        
    depth++;
    
    if( node.hasChildNodes() ) {
      NodeList nodeList = node.getChildNodes();
      for( int i=0; i < nodeList.getLength() ; i++) {
        printDOMTree(nodeList.item(i));
        depth--;
      }      
    }
  }

  
  protected void ParseXML() throws ECfgReaderCGeneralError {
    //Instantiate the Factory
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    // activate Validating parser
    factory.setValidating(true);
    // activate Namespaces
    factory.setNamespaceAware(true);
    try {
      //Get a Parser and Parse the File
      DocumentBuilder builder = factory.newDocumentBuilder();
      document = builder.parse( new File(m_XMLSourceFile) );
      DomNode = document;   
      // print out the Document data
      //XmlDocument xdoc = (XmlDocument) document;
      //xdoc.write (System.out);            
            
      //document = factory.newDocument();
            
      //DomNode = document;   
      //printDOMTree(DomNode);
      
      } catch (SAXParseException spe) {
        // Error generated by the parser
        // Use the contained exception, if any
        Exception  x = spe;
        if (spe.getException() != null)
          x = spe.getException();
        x.printStackTrace();
        throw new ECfgReaderCGeneralError("\n** Parsing error" + ", line " + spe.getLineNumber ()  + "\n"
                                          + ", uri " + spe.getSystemId () + "\n" 
                                          + spe.getMessage() + "\n"
                                          + spe.toString() + "\n" + this.toString());
      } catch (SAXException sxe) {
        // Error generated by this application
        // (or a parser-initialization error)
        Exception  x = sxe;
        if (sxe.getException() != null)
          x = sxe.getException();
        x.printStackTrace();
        throw new ECfgReaderCGeneralError("SAXException " + sxe.toString() +" "+ "\n" + this.toString());
      } catch (ParserConfigurationException pce) {
        // Parser with specified options can't be built
        pce.printStackTrace();
        throw new ECfgReaderCGeneralError("Parser with specified options can't be built ! " + "\n" + pce.toString() + "\n" + this.toString());
      } catch (IOException ioe) {
        // I/O error
        ioe.printStackTrace();
        throw new ECfgReaderCGeneralError("File "+ m_XMLSourceFile +" can't be found ! " + "\n" + ioe.toString() + "\n" + this.toString()); 
      }
  }
  //===========================================================
  // SAX ErrorHandler methods
  //===========================================================

  // treat validation errors as fatal
  public void error (SAXParseException e)
  throws SAXParseException
  {
      throw e;
  }
/*
  // dump warnings too
  public void warning (SAXParseException err)
  throws SAXParseException
  {
      System.out.println ("** Warning"
          + ", line " + err.getLineNumber ()
          + ", uri " + err.getSystemId ());
      System.out.println("   " + err.getMessage ());
  }*/
  
  /*
  public static void main (String args[]) {
    YXMLReader XMLr = new YXMLReader();
    
    try {
      XMLr.SetXMLSource("C:\\SOURCE\\java\\YARB\\YARB\\XML\\ConfigFileSample.xml");   
            
      XMLr.ParseXML();
  
      Node node = XMLr.FindNode("YARBConfiguration");
      XMLr.SetCurrentNode(node);      
      System.out.println( XMLr.getElement("Title") );
      
      node = XMLr.FindNode("FileSources");      
      XMLr.SetCurrentNode(node);
      System.out.println( XMLr.getElement("Class") );
      
      node = XMLr.FindNode("DataFilter");      
      XMLr.SetCurrentNode(node);
      System.out.println( XMLr.getElement("Class") );
      
      node = XMLr.FindNode("DataFilter");            
      XMLr.SetCurrentNode(node);
      System.out.println( XMLr.getElement("Class") );

      node = XMLr.FindNode("DataViewer");      
      XMLr.SetCurrentNode(node);
      System.out.println( XMLr.getElement("Class") );
          
    } catch(YXMLReader.ECfgReaderCGeneralError e) {
      System.out.println( e.toString() );
    }
  
  }*/
  
  //***
  //*** Deklaration of the exception classes:
  //***

  /**
  * general error exception class
  */
  class ECfgReaderCGeneralError extends Exception
  {
    ECfgReaderCGeneralError(String description)
    {
      super(description);
    }
  }
  
}