/*
 * YXMLWriter.java
 *
 * Created on 4. Juni 2000, 20:42
 */
 
package YARB.XML;

import java.io.*;


/** 
 *
 * @author  Administrator
 * @version 
 */
public class YXMLWriter extends Object {
  // Source file
  private String m_XMLSourceFile = null;
  // Tree with XML data
  private String[] m_nodeTree = null;
  private int m_InodeTreeLength = 0;
  // Tree with depth of each node
  private int[] m_iDepthTree = null;
  private int m_iDepth = 0;
  // Tree for tracking nodes
  private String[] m_TrackOpenNodes = null;
  private int m_ITrackOpenNodesLength = 0;
  
  /** Creates new YXMLWriter */
  public YXMLWriter() {
  }

  /** Set XML Source File */
  public void SetXMLSource(String src) {
    m_XMLSourceFile = src;
  }
  // Valid Elements in XML File
  static final String[] m_validElements = {
    //"YARBMultipleConfiguration", "YARBSingleConfiguration",
    "YARBConfiguration",
    "DataFilter", "FileSources", "DataViewer",
    "Title", "Class", "File",
    "boolean", "int", "float", "String",
  };
  // YARB DTD
  static final String m_DTD_FileName= "YARBConfiguration.dtd";
  static final String[] m_YARB_DTD = {
    "<?xml version='1.0' encoding='UTF-8'?>",
    "<!ELEMENT YARBConfiguration (Title,File,FileSources, DataFilter*, DataViewer)>",
    "<!ELEMENT DataFilter (Title, Class, boolean*, int*, float*, String*)>",
    "<!ELEMENT FileSources (Title, Class,boolean*, int*, float*, String* )>",
    "<!ELEMENT DataViewer (Title, Class, boolean*, int*, float*, String*)>",    
    "<!ELEMENT boolean (#PCDATA) >",
    "<!ELEMENT int (#PCDATA) >",
    "<!ELEMENT float (#PCDATA) >",
    "<!ELEMENT String (#PCDATA) >",
    "<!ELEMENT File (#PCDATA) >",
    "<!ELEMENT Class (#PCDATA) >",
    "<!ELEMENT Title (#PCDATA) >",
  };
  // Amount of valid elements in XML File
  static final int m_IvalidElements = 11;

  public void ResetAll() {
    // Source file
    m_XMLSourceFile = null;
    // Tree with XML data
    m_nodeTree = null;
    m_InodeTreeLength = 0;
    // Tree for tracking nodes
    m_TrackOpenNodes = null;
    int m_ITrackOpenNodesLength = 0;    
  }
  
  private boolean isElementValid(String ElementName) {
    for( int i=0; i < m_IvalidElements; i++) {
      if( ElementName.compareTo(m_validElements[i]) == 0 ) return true;
    }
    return false;
  }
  
  public void printNodeTree() {    
    for( int i=0; i < m_InodeTreeLength; i++) System.out.println(m_nodeTree[i]);
  }
  public void printTrack() {
    for( int i=0; i < m_ITrackOpenNodesLength; i++) System.out.println(m_TrackOpenNodes[i]);    
  }

  /** Creates DTD file in same directory as the configuration */
  private void CreateDTD()  throws ECgfWriterCGeneralError {
    if( m_XMLSourceFile == null || m_XMLSourceFile.length() == 0 ) {
      throw new ECgfWriterCGeneralError( "Null pointer occured !" + "\n" + this.toString() );
    }
    //m_XMLSourceFile
    File Config = new File(m_XMLSourceFile) ;
    File DTD = new File( Config.getParent() +"/"+ m_DTD_FileName );
    if( DTD.exists() ) {      
      return;
    }
    System.out.println("Creating DTD file <" + DTD.getPath() + ">!");
    try {
      BufferedWriter DTDSrc = new BufferedWriter( new FileWriter( DTD.getPath() ) );
      
      for( int i=0; i < m_YARB_DTD.length; i++) {
        DTDSrc.write(m_YARB_DTD[i]);
        DTDSrc.newLine();
      }
      
      DTDSrc.close();
      
    } catch (IOException e) {
      throw new ECgfWriterCGeneralError( "Can't create DTD file!" + e.toString() + "\n" + this.toString() );
    }
  }

  /** Write XML data out to a file */
  
  public void writeXMLFile() throws ECgfWriterCGeneralError {    
    if( m_ITrackOpenNodesLength != 0 ) {
      throw new ECgfWriterCGeneralError("Can't create XML file, not all nodes have been closed ! " + "\n" + this.toString());
    }
    try {
      BufferedWriter OutputSrc = new BufferedWriter( new FileWriter(m_XMLSourceFile) );
      
      // Header
      OutputSrc.write("<?xml version='1.0' encoding='UTF-8'?>");
      OutputSrc.newLine();
      OutputSrc.write("<!DOCTYPE YARBConfiguration SYSTEM \"YARBConfiguration.dtd\" [] >");
      OutputSrc.newLine();
      // NodeTree
      for( int i=0; i < m_InodeTreeLength; i++) {
        OutputSrc.write(m_nodeTree[i]);
        if( m_nodeTree[i].startsWith("</") ) OutputSrc.newLine();
        else {
          if( m_nodeTree[i].charAt(0) == '<' && m_nodeTree[i+1].charAt(0) == '<' ) OutputSrc.newLine();
        }
        
      }
      
      OutputSrc.close();
      CreateDTD();
    } catch (IOException e) {
      System.out.println("***IOException: " + e.toString());
    }          
  
  }
  
  /** Add node to tree */
  public void addNode(String NodeName) throws ECgfWriterCGeneralError {
    if( NodeName == null ) {
      throw new ECgfWriterCGeneralError("Null pointer occured ! " + "\n" + this.toString() );
    }
    if( !isElementValid(NodeName) ) {
      throw new ECgfWriterCGeneralError(NodeName + " isn't a valid node for YARB! " + "\n" + this.toString() );
    }
    // format string to <NodeName>    
    StringBuffer tmp = new StringBuffer("<");
    tmp.insert(tmp.length(), NodeName);
    tmp.append(">");
    // add to NodeTree
    if( m_InodeTreeLength == 0 ) {
      // first node
      if( NodeName.compareTo("YARBConfiguration") != 0 )
        throw new ECgfWriterCGeneralError("First node in Tree must be <YARBConfiguration> ! "+"\n" + this.toString());
      m_InodeTreeLength++;
      m_ITrackOpenNodesLength++;      
      m_nodeTree = new String[1];
      m_nodeTree[0] = tmp.toString();
      m_TrackOpenNodes = new String[1];
      m_TrackOpenNodes[0] = NodeName;
      /*
      m_iDepthTree = new int[1];
      m_iDepthTree[0] = m_iDepth;
      m_iDepth++;
      */
      return ;
    }
    // Save old nodeTree
    String[] tmpTree = m_nodeTree;
    // Create new nodeTree
    m_InodeTreeLength++;
    m_nodeTree = new String[m_InodeTreeLength];
    for( int i=0; i < m_InodeTreeLength-1; i++)
      m_nodeTree[i] = tmpTree[i];         
    m_nodeTree[m_InodeTreeLength-1] = tmp.toString();
    /*
    // depth tracking of each node
    int depth[] = m_iDepthTree;
    
    if( NodeName.compareTo("FileSources") || NodeName.compareTo("DataFilter") || NodeName.compareTo("DataViewer") ) {
      m_iDepth++;
    }*/
    
    // Save old tracking
    tmpTree = m_TrackOpenNodes;
    // Create new nodeTree
    m_ITrackOpenNodesLength++;
    m_TrackOpenNodes = new String[m_ITrackOpenNodesLength];
    for( int i=0; i < m_ITrackOpenNodesLength-1; i++)
      m_TrackOpenNodes[i] = tmpTree[i];         
    m_TrackOpenNodes[m_ITrackOpenNodesLength-1] = NodeName;
    
    
    
  }
  
  /** Close node from tree */
  public void closeNode(String NodeName) throws ECgfWriterCGeneralError {
    if( NodeName == null ) {
      throw new ECgfWriterCGeneralError("Null pointer occured ! " + "\n" + this.toString() );
    }    
    if( !isElementValid(NodeName) ) {
      throw new ECgfWriterCGeneralError(NodeName + " isn't a valid node for YARB! " + "\n" + this.toString() );
    }
      
    if( m_ITrackOpenNodesLength == 0 ) {
      throw new ECgfWriterCGeneralError("No node is open to close ! " + "\n" + this.toString());
    }
    // node to close must be last in list
    if( NodeName.compareTo(m_TrackOpenNodes[m_ITrackOpenNodesLength-1]) != 0 ) {
      throw new ECgfWriterCGeneralError("Before node <"+NodeName+"> other nodes have to be closed first ! (last = " + m_TrackOpenNodes[m_ITrackOpenNodesLength-1] + ")" + "\n" + this.toString());
    }
    // Delete last element
    m_ITrackOpenNodesLength--;
    String[] tmpTree = m_TrackOpenNodes;
    m_TrackOpenNodes = new String[m_ITrackOpenNodesLength];
    for( int i=0; i < m_ITrackOpenNodesLength; i++) m_TrackOpenNodes[i] = tmpTree[i];
    // Add to NodeTree

    // format string to </NodeName>    
    StringBuffer tmp = new StringBuffer("</");
    tmp.insert(tmp.length(), NodeName);
    tmp.append(">");

    // Save old nodeTree
    tmpTree = m_nodeTree;
    // Create new nodeTree
    m_InodeTreeLength++;
    m_nodeTree = new String[m_InodeTreeLength];
    for( int i=0; i < m_InodeTreeLength-1; i++)
      m_nodeTree[i] = tmpTree[i];         
    m_nodeTree[m_InodeTreeLength-1] = tmp.toString();
    
    // m_ITrackOpenNodesLength triggers write out if 0
    /*if(m_ITrackOpenNodesLength == 0) {
      writeXMLFile();
    }*/
    
  }
   
  /** Replace forbidden character with valid XML Code
    * & := &amp;
    * < := &lt;
    * > := &gt;
    * " := &quot; 
    * ' := &apos;
   */
  String FormatXMLString(String str) {
    if( str == null || str.length() == 0 ) return str;
    StringBuffer strb = new StringBuffer("");
    for(int i=0; i < str.length(); i++) {
      
      if( str.charAt(i) == '&' )  strb.append("&#38;");      //strb.append("&amp;");      
      else {
        if( str.charAt(i) == '<' )  strb.append("&#60;");      //strb.append("&lt;");        
        else {
          if( str.charAt(i) == '>' )  strb.append("&#62;");      //strb.append("&gt;");          
          else {
            if( str.charAt(i) == '\"' ) strb.append("&#34;");      //strb.append("&quot;");            
            else {
              if( str.charAt(i) == '\'' ) strb.append("&apos;");              
              else {
                if( str.charAt(i) == '' )  strb.append("ss");
                else {
                  if( str.charAt(i) == '' )  strb.append("ae");
                  else {
                    if( str.charAt(i) == '' )  strb.append("oe");
                    else {
                      if( str.charAt(i) == '' ) strb.append("ue");
                      else {
                        strb.append(str.charAt(i));
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }    
    return strb.toString();
  }
  
  /** Add element to tree */
  public void addElement(String ElementName, String ElementValue) throws ECgfWriterCGeneralError {
    if( ElementName == null ) {
      throw new ECgfWriterCGeneralError("Null pointer occured ! " + "\n" + this.toString() );
    }
        
    if( !isElementValid(ElementName) ) {
      throw new ECgfWriterCGeneralError(ElementName + " isn't a valid node for YARB! " + "\n" + this.toString() );
    }      
    try {
      if( ElementName != null ) addNode(ElementName);
      else addNode("none");
      // add value to tree
      // Save old nodeTree
      String [] tmpTree = m_nodeTree;
      // Create new nodeTree
      m_InodeTreeLength++;
      m_nodeTree = new String[m_InodeTreeLength];
      for( int i=0; i < m_InodeTreeLength-1; i++)
        m_nodeTree[i] = tmpTree[i];         
      m_nodeTree[m_InodeTreeLength-1] = FormatXMLString(ElementValue) ;
      
      closeNode(ElementName);
    } catch(ECgfWriterCGeneralError e) {
      throw new ECgfWriterCGeneralError( e.toString()  + "\n" + this.toString() );
    }
  }  
/*    
  public static void main (String args[]) {
    YXMLWriter test = new YXMLWriter();
    System.out.println( test.FormatXMLString(" und & ; left < ; right > ; quote \" ; other ' ; ue  ; ae  ;  oe; ss ") );
    //test.SetXMLSource("d:\\test.xml");
    
    try {
      test.addNode("YARBConfiguration");
      test.addElement("Title","YARB");
      test.addNode("FileSources");
      test.addElement("Class",test.toString());
      test.addElement("Title","Pattern");
      test.addElement("File","d:\\examples\\xor.pat");
      test.addElement("Class","YARB.FileSources.YFSSNNSPatternFiles");
      test.addElement("boolean","true");
      test.addElement("int","12345");
      test.closeNode("FileSources");
      test.closeNode("YARBConfiguration");
      //test.printNodeTree();
      //test.printTrack();
      test.writeXMLFile();

      test.CreateDTD();
    } catch(ECgfWriterCGeneralError e) {
      System.out.println(e.toString() );
    }
  }*/
  
  /**
  * general error exception class for this interface
  */
  class ECgfWriterCGeneralError extends Exception
  {
    ECgfWriterCGeneralError(String description)
    {
      super(description);
    }
  }  
  
}